@@ -466,7 +466,7 @@ void key_unpack(String *to, TABLE *table, KEY *key)
 
 bool is_key_used(TABLE *table, uint idx, const MY_BITMAP *fields)
 {
-  table->mark_columns_used_by_index(idx, &table->tmp_set);
+  table->mark_index_columns(idx, &table->tmp_set);
   return bitmap_is_overlapping(&table->tmp_set, fields);
 }
 
