@@ -328,7 +328,7 @@ static dberr_t create_log_file(bool create_new_db, lsn_t lsn,
 	mysql_mutex_unlock(&log_sys.mutex);
 
 	log_make_checkpoint();
-	log_write_up_to(LSN_MAX, true);
+	log_buffer_flush_to_disk();
 
 	return DB_SUCCESS;
 }
@@ -812,9 +812,9 @@ srv_open_tmp_tablespace(bool create_new_db)
 static void srv_shutdown_threads()
 {
 	ut_ad(!srv_undo_sources);
-	srv_shutdown_state = SRV_SHUTDOWN_EXIT_THREADS;
 	ut_d(srv_master_thread_enable());
 	srv_master_timer.reset();
+	srv_shutdown_state = SRV_SHUTDOWN_EXIT_THREADS;
 
 	if (purge_sys.enabled()) {
 		srv_purge_shutdown();
